from sys import path
import yaml
import pathlib

SETTINGS = {
    "show_command": True,
    "show_reason": True,
    "show_output": True,
    "colour": True
}

def load_settings(filepath):
    filepath = pathlib.Path(filepath)

    with filepath.open("r") as f:
        SETTINGS.update(
            yaml.safe_load(f.read())
        )

def setup_settings(filepath):
    root = pathlib.Path(filepath)
    config = pathlib.Path(root, "shortcut.d")

    root.mkdir()
    config.mkdir()

    with pathlib.Path(root, "settings.yaml").open("w") as f:
        f.write(
            yaml.safe_dump(SETTINGS, indent=2)
        )


from .exceptions import RequiredValue

__author__ = "Matt Limb <matt.limb17@gmail.com>"
class Option:
    TYPES = dict(
        string=str,
        integer=int,
        float=float
    )

    def __init__(self, **kwargs):
        self.name = kwargs.get("name", None)
        self.description = kwargs.get("description", None)
        self.short = kwargs.get("short", None)
        self.long = kwargs.get("long", None) 
        self.arg_type = kwargs.get("arg_type", "flag")

        self.default = kwargs.get("default", None)
        self.type = self.TYPES[kwargs.get("type", "string")]
        self.required = kwargs.get("required", False)

        self._verify()

    def _verify(self):
        if not self.name:
            raise RequiredValue("'name' is a required value of an option.")
        
    @staticmethod
    def new(name, conf):
        conf["name"] = name
        return Option(**conf)

    def add_option(self, parser=None):
        option_args = list()
        option_kwargs = dict()

        if self.short:
            if self.short[0] != "-":
                option_args = f"-{self.short}"
            else:
                option_args.append(self.short)
        
        if self.long:
            num_dash = self.long.count("-")
            if num_dash >= 2:
                option_args.append(self.long)
            elif num_dash == 1:
                option_args.append(f"-{self.long}")
            else:
                option_args.append(f"--{self.long}")

        option_kwargs["dest"] = self.name
        
        option_kwargs["required"] = self.required
        
        if self.description:
            option_kwargs["help"] = self.description

        if self.arg_type.lower() == "flag":
            option_kwargs["action"] = "store_true"

        elif self.arg_type.lower() == "data":
            option_kwargs["action"] = "store" 
            option_kwargs["default"] = self.default
            option_kwargs["type"] = self.type
        
        parser.add_argument(*option_args, **option_kwargs)
            
        return parser

    def __repr__(self):
        """ String representation of the class """
        items = []

        for k, v in self.__dict__.items():
            if "_" != k[0]:
                if "pass" in k:
                    v = '*' * len(v)
                
                if isinstance(v, str):
                    items.append(f"{k}='{v}'")
                else:
                    items.append(f"{k}={v}")

        items = ', '.join(items)

        return f"{self.__class__.__name__}({items})"


from sys import path
from .exceptions import RequiredValue
from .options import Option
from .commands import Command
import pathlib
import yaml

__author__ = "Matt Limb <matt.limb17@gmail.com>"

class Shortcut:
    def __init__(self, **kwargs):
        self.name = kwargs.get("name", None)
        self.description = kwargs.get("description", None)
        self.cmd = kwargs.get("cmd", None)

        self.options = kwargs.get("options", {})
        self.commands = kwargs.get("commands", {})

        self._verify()

    def _verify(self):
        if not self.name:
            raise RequiredValue("'name' is a required value of a shortcut.")

        if not self.description:
            raise RequiredValue("'description' is a required value of a shortcut.")

        if not self.cmd:
            raise RequiredValue("'cmd' is a required value of a shortcut.")

    @staticmethod
    def new(file_contents):
        file_contents["options"] = [
            Option.new(name, option) 
                for name, option in file_contents.get("options", {}).items()
        ]

        if file_contents["commands"]:
            file_contents["commands"] = [
                Command.new(name, conf)
                    for name, conf in file_contents.get("commands", {}).items()
            ]
        else:
            raise RequiredValue("'commands' are a required value of a shortcut.")

        return Shortcut(**file_contents)

    @staticmethod
    def from_file(filepath):
        filepath = pathlib.Path(filepath)
        shortcusts = []

        with filepath.open("r") as f:
            file_contents = yaml.safe_load_all(f.read())

        for content in file_contents:
            content["options"] = [
                Option.new(name, option) 
                    for name, option in content.get("options", {}).items()
            ]

            if content["commands"]:
                content["commands"] = [
                    Command.new(name, conf)
                        for name, conf in content.get("commands", {}).items()
                ]
            else:
                raise RequiredValue("'commands' are a required value of a shortcut.")

            shortcusts.append(Shortcut(**content))
        
        return shortcusts


    def run_commands(self, variables):
        variables = {
            "option": vars(variables),
            "command": {} 
        }

        for command in self.commands:
            rc, output = command.run_command(variables)

            variables["command"][command.name] = {
                "return": rc,
                "output": output
            }

    def add_parser(self, parser=None):
        p = parser.add_parser(self.cmd, help=self.description)

        for option in self.options:
            option.add_option(p)
        
        return parser

    def __repr__(self):
        """ String representation of the class """
        items = []

        for k, v in self.__dict__.items():
            if "_" != k[0]:
                if "pass" in k:
                    v = '*' * len(v)
                
                if isinstance(v, str):
                    items.append(f"{k}='{v}'")
                else:
                    items.append(f"{k}={v}")

        items = ', '.join(items)

        return f"{self.__class__.__name__}({items})"



import argparse
import os
import pathlib
from colorama import init
from . import SETTINGS, load_settings, setup_settings

from .shortcut import Shortcut

__author__ = "Matt Limb <matt.limb17@gmail.com>"

def main():
    # Find Home Directory
    home = pathlib.Path(pathlib.Path.home(), "shortcut")

    # Setup Setting Path
    setting_config = pathlib.Path(os.environ.get("SHORTCUT_CONFIG", str(home)))

    # Setup Config if it doesn't exist
    if not setting_config.exists():
        setup_settings(setting_config)

    # Load settings from file
    load_settings(pathlib.Path(setting_config, "settings.yaml"))

    # Setup colour is enabled
    if SETTINGS["colour"]:
        init()
    
    # Setup Command Dict
    commands = dict()

    # Setup the commands
    for config in pathlib.Path(setting_config, "shortcut.d").glob("*.yaml"):
        s = Shortcut.from_file(config)

        for x in s:
            commands[x.cmd] = x    
        
    # Setup CLI parser
    parser = argparse.ArgumentParser(prog="shortcut")
    sub_parser = parser.add_subparsers(help="Commands", dest="command")

    # Register commands with CLI
    for _, cmd in commands.items():
        cmd.add_parser(sub_parser)

    # Parse CLI    
    args = parser.parse_args()

    if args.command == None:
        args = parser.parse_args([ "--help" ])

    # Run the Command
    commands[args.command].run_commands(args)

if __name__ == "__main__":
    main()


from .exceptions import RequiredValue
from . import SETTINGS
from termcolor import colored
from colorama import Style
import shlex
import subprocess

__author__ = "Matt Limb <matt.limb17@gmail.com>"

class Command:
    def __init__(self, **kwargs):
        self.name = kwargs.get("name", None)
        self.description = kwargs.get("description", None)
        self.cmd = kwargs.get("cmd", None)
        self.conditionals = kwargs.get("if", [])

        self._verify()

    def _verify(self):
        if not self.name:
            raise RequiredValue("'name' is a required value of a command.")

        if not self.cmd:
            raise RequiredValue("'cmd' is a rrequired value of a command")

        if isinstance(self.cmd, str):
            self.cmd = shlex.split(self.cmd)

    def _replace_var(self, var, variables, context="variable"):
        if ":" in var:
            pre, post = var.split(":")

            if pre in [ "option", "command" ]:

                if post in variables[pre]:
                    tvar = variables[pre][post]
                    
                    if context == "variable":
                        if isinstance(tvar, dict):
                            return tvar["output"]
                        else:
                            return tvar
                    elif context == "conditional":
                        return tvar
                    
            return var

        return var

    def _process_conditional(self, name, value, conditional):
        
        if "eq" in conditional.keys():
            if value == conditional["eq"]:
                return ( True, f"{value} ({name}) equal to {conditional['eq']}")
            else:
                return ( False, f"{value} ({name}) not equal to {conditional['eq']}")
        
        if "neq" in conditional.keys():
            if value != conditional["neq"]:
                return ( True, f"{value} ({name}) not equal to {conditional['neq']}")
            else:
                return ( False, f"{value} ({name}) equal to {conditional['neq']}")
        
        
        if isinstance(value, (int, float)):
            if "gt" in conditional.keys():
                if value > conditional["gt"]:
                    return ( True, f"{value} ({name}) greater than {conditional['gt']}")
                else:
                    return ( False, f"{value} ({name}) less than or equal to {conditional['gt']}")

            if "ge" in conditional.keys():
                if value >= conditional["ge"]:
                    return ( True, f"{value} ({name}) greater than or equal to {conditional['ge']}")
                else:
                    return ( False, f"{value} ({name}) less than {conditional['ge']}")
            
            if "lt" in conditional.keys():
                if value < conditional["lt"]:
                    return ( True, f"{value} ({name}) less than {conditional['lt']}")
                else:
                    return ( False, f"{value} ({name}) greater than or equal to {conditional['lt']}")

            if "le" in conditional.keys():
                if value <= conditional["le"]:
                    return ( True, f"{value} ({name}) less than or equal to {conditional['le']}")
                else:
                    return ( False, f"{value} ({name}) greater than {conditional['le']}")

        if isinstance(value, dict):
            if "return" in conditional.keys():
                if value["return"] == conditional["return"]:
                    return ( True, f"{name} returned {conditional['return']}")
                else:
                    return ( False, f"{name} returned {conditional['return']}")

            if "includes" in conditional.keys():
                if f"{conditional['includes']}\n" in value["output"]:
                    return ( True, f"{value['includes']} in output of {name}")
                else:
                    return ( False, f"{value['includes']} in output of {name}")

        return ( False, f"{value} ({name}) is an unsupported type")
        

    def can_run(self, variables):
        if len(self.conditionals) == 0:
            return ( True, [ "No Conditionals" ] )
        else:
            success = []
            failure = []
            for name, conditional in self.conditionals.items():
                val = self._replace_var(name, variables, context="conditional")
                
                des, message = self._process_conditional(name, val, conditional)
                
                if des:
                    success.append(message)
                else:
                    failure.append(message)

            if len(failure) > 0:
                return ( False, failure )
            else:
                return ( True, success )
        
    
    def run_command(self, variables):
        run, messages = self.can_run(variables)

        if run:
            if SETTINGS["show_command"]:
                self.output_to_term(f"----------------------\nRunning {self.name}")
            
            if SETTINGS["show_reason"]:
                if not SETTINGS["show_command"]:
                    self.output_to_term("----------------------\nReason: {}".format(", ".join(messages)))
                else:
                    self.output_to_term("Reason: {}".format(", ".join(messages)))
            
            for c, pt in enumerate(self.cmd):
                self.cmd[c] = self._replace_var(pt, variables)
            
            data = []
            if SETTINGS["show_output"]:
                if ( not SETTINGS["show_command"] ) and ( not SETTINGS["show_reason"] ):
                    self.output_to_term(f"----------------------\nOutput\n----------------------")
                else:
                    self.output_to_term(f"Output\n----------------------")

            sp = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE)

            with sp as out:
                data.append(out.stdout.read().decode("utf-8").strip())
                if SETTINGS["show_output"]:
                    print(data[-1])
            
            if SETTINGS["show_output"]:
                print()
            
            return ( sp.returncode, "\n".join(data) )

        else:
            if SETTINGS["show_command"]:
                self.output_to_term(f"----------------------\nSkipping {self.name}")
            
            if SETTINGS["show_command"]:
                self.output_to_term("Reason: {}".format(", ".join(messages)))
            
            if SETTINGS["show_output"]:
                self.output_to_term("----------------------\n")

            return ( 999, "" )

    def output_to_term(self, message):
        if SETTINGS["colour"]:
            print(colored(message, "green"))
            Style.RESET_ALL
        else:
            print(message)
    
    @staticmethod
    def new(name, conf):
        conf["name"] = name
        return Command(**conf)

    def __repr__(self):
        """ String representation of the class """
        items = []

        for k, v in self.__dict__.items():
            if "_" != k[0]:
                if "pass" in k:
                    v = '*' * len(v)
                
                if isinstance(v, str):
                    items.append(f"{k}='{v}'")
                else:
                    items.append(f"{k}={v}")

        items = ', '.join(items)

        return f"{self.__class__.__name__}({items})"


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    require = fh.readlines()

setuptools.setup(
    name="shortcut-alias", # Replace with your own username
    version="0.1.0",
    author="Matt Limb",
    author_email="matt.limb17@gmail.com",
    description="Powerful Configurable Aliases",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        req.strip() 
        for req in require
    ],
    python_requires='>=3.6',
    entry_points={  
        'console_scripts': [
            'shortcut-alias=shortcut_alias.__main__:main',
            'shortcut=shortcut_alias.__main__:main',
            'sa=shortcut_alias.__main__:main',
        ],
    }
)